struct VS_INPUT
{
	float4 position : POSITION;
	float3 normal	: NORMAL;
};

struct VS_OUTPUT
{
	float4 position	: POSITION;
	float4 oPosition : TEXCOORD0;
	float3 normal	: TEXCOORD1;
};

struct PS_INPUT
{
	float4 position : TEXCOORD0;
	float3 normal	: TEXCOORD1;
};

struct PS_OUTPUT
{
	float4 color	: COLOR;
};

VS_OUTPUT main_vp(
		VS_INPUT input,
		uniform float4x4 worldViewProj)
{
	VS_OUTPUT output;

	output.position = mul(worldViewProj, input.position);
	output.oPosition = input.position;
	output.normal = input.normal;

	return output;
}

PS_OUTPUT main_fp(PS_INPUT input,
		uniform float4 ambientColour,
		uniform float4 lightPosition)
{
	PS_OUTPUT output;

	float3 n = normalize(input.normal);
	float3 l = normalize(lightPosition.xyz - input.position.xyz);

	float NdotL = dot(n, l);
	float3 baseColor = float3(1.0, 1.0, 1.0);

	/*
	float3 baseColor = float3(0.0, 0.0, 1.0);
	if (input.position.y > 0)
	{
		baseColor = float3(1.0, 1.0, 1.0);
	}
	*/

	float3 diffuse = NdotL * baseColor;
	diffuse *= (float3(1.0, 1.0, 1.0) - ambientColour.rgb);

	output.color.rgb = ambientColour + diffuse;
	output.color.a = 1.0;

	return output;
}